import json
import unittest
from unittest import TestCase, mock

import pandas as pd

from ..FOMC_Statement.statements import Statements


class Test_Statements(TestCase):
    """Testing FOMC Statement Methods"""

    def test_make_endpoint_query(self) -> None:
        """Test method responsible for making endpoint queries."""
        url_base = "http://18.220.119.101:8080/fed/get-fomc-statement/"
        scenarios: list = [
            [None, url_base],
            ["current", url_base + "current"],
            [["2009-03-18"], url_base + "2009-03-18"],
            [["current", "previous"], url_base + "current,previous"],
            [["current", "previous", "2020-03-03"], url_base + "current,previous,2020-03-03"],
        ]
        for scenario in scenarios:
            result = Statements(scenario[0]).make_endpoint_query()
            expected = scenario[-1]
            self.assertEqual(result, expected)

    def test_fetch_data(self) -> None:
        """Test method responsible for fetching data from API."""
        scenario = "2020-04-29"
        expected_snippet = "Weaker demand and significantly lower oil prices are holding down consumer price inflation."
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'{"2020-04-29":"The Federal Reserve is committed to using its full range of tools to support the U.S. economy in this challenging time, thereby promoting its maximum employment and price stability goals.\n\n      The coronavirus outbreak is causing tremendous human and economic hardship across the United States and around the world. The virus and the measures taken to protect public health are inducing sharp declines in economic activity and a surge in job losses. Weaker demand and significantly lower oil prices are holding down consumer price inflation. The disruptions to economic activity here and abroad have significantly affected financial conditions and have impaired the flow of credit to U.S. households and businesses.\n      \n      The ongoing public health crisis will weigh heavily on economic activity, employment, and inflation in the near term, and poses considerable risks to the economic outlook over the medium term. In light of these developments, the Committee decided to maintain the target range for the federal funds rate at 0 to 1/4 percent. The Committee expects to maintain this target range until it is confident that the economy has weathered recent events and is on track to achieve its maximum employment and price stability goals.\n      \n      The Committee will continue to monitor the implications of incoming information for the economic outlook, including information related to public health, as well as global developments and muted inflation pressures, and will use its tools and act as appropriate to support the economy. In determining the timing and size of future adjustments to the stance of monetary policy, the Committee will assess realized and expected economic conditions relative to its maximum employment objective and its symmetric 2 percent inflation objective. This assessment will take into account a wide range of information, including measures of labor market conditions, indicators of inflation pressures and inflation expectations, and readings on financial and international developments.\n      \n      To support the flow of credit to households and businesses, the Federal Reserve will continue to purchase Treasury securities and agency residential and commercial mortgage-backed securities in the amounts needed to support smooth market functioning, thereby fostering effective transmission of monetary policy to broader financial conditions. In addition, the Open Market Desk will continue to offer large-scale overnight and term repurchase agreement operations. The Committee will closely monitor market conditions and is prepared to adjust its plans as appropriate.\n      \n      Voting for the monetary policy action were Jerome H. Powell, Chair; John C. Williams, Vice Chair; Michelle W. Bowman; Lael Brainard; Richard H. Clarida; Patrick Harker; Robert S. Kaplan; Neel Kashkari; Loretta J. Mester; and Randal K. Quarles."}'  # noqa: E501
            result = Statements(scenario).fetch_data()
            self.assertIn(expected_snippet, result)
            self.assertIsInstance(result, str)

    def test_to_DataFrame(self) -> None:
        """Test method responsible for converting data to DataFrame."""
        demo = json.dumps({"2021-11-03": "The Federal Reserve is committed to using its full range of tools to support the U.S. economy in this challenging time, thereby promoting its maximum employment and price stability goals.\n\n      With progress on vaccinations and strong policy support, indicators of economic activity and employment have continued to strengthen. The sectors most adversely affected by the pandemic have improved in recent months, but the summer's rise in COVID-19 cases has slowed their recovery. Inflation is elevated, largely reflecting factors that are expected to be transitory. Supply and demand imbalances related to the pandemic and the reopening of the economy have contributed to sizable price increases in some sectors. Overall financial conditions remain accommodative, in part reflecting policy measures to support the economy and the flow of credit to U.S. households and businesses.\n      \n      The path of the economy continues to depend on the course of the virus. Progress on vaccinations and an easing of supply constraints are expected to support continued gains in economic activity and employment as well as a reduction in inflation. Risks to the economic outlook remain.\n      \n      The Committee seeks to achieve maximum employment and inflation at the rate of 2 percent over the longer run. With inflation having run persistently below this longer-run goal, the Committee will aim to achieve inflation moderately above 2 percent for some time so that inflation averages 2 percent over time and longer term inflation expectations remain well anchored at 2 percent. The Committee expects to maintain an accommodative stance of monetary policy until these outcomes are achieved. The Committee decided to keep the target range for the federal funds rate at 0 to 1/4 percent and expects it will be appropriate to maintain this target range until labor market conditions have reached levels consistent with the Committee's assessments of maximum employment and inflation has risen to 2 percent and is on track to moderately exceed 2 percent for some time. In light of the substantial further progress the economy has made toward the Committee's goals since last December, the Committee decided to begin reducing the monthly pace of its net asset purchases by $10 billion for Treasury securities and $5 billion for agency mortgage-backed securities. Beginning later this month, the Committee will increase its holdings of Treasury securities by at least $70 billion per month and of agency mortgage backed securities by at least $35 billion per month. Beginning in December, the Committee will increase its holdings of Treasury securities by at least $60 billion per month and of agency mortgage-backed securities by at least $30 billion per month. The Committee judges that similar reductions in the pace of net asset purchases will likely be appropriate each month, but it is prepared to adjust the pace of purchases if warranted by changes in the economic outlook. The Federal Reserve's ongoing purchases and holdings of securities will continue to foster smooth market functioning and accommodative financial conditions, thereby supporting the flow of credit to households and businesses.\n      \n      In assessing the appropriate stance of monetary policy, the Committee will continue to monitor the implications of incoming information for the economic outlook. The Committee would be prepared to adjust the stance of monetary policy as appropriate if risks emerge that could impede the attainment of the Committee's goals. The Committee's assessments will take into account a wide range of information, including readings on public health, labor market conditions, inflation pressures and inflation expectations, and financial and international developments.\n      \n      Voting for the monetary policy action were Jerome H. Powell, Chair; John C. Williams, Vice Chair; Thomas I. Barkin; Raphael W. Bostic; Michelle W. Bowman; Lael Brainard; Richard H. Clarida; Mary C. Daly; Charles L. Evans; Randal K. Quarles; and Christopher J. Waller."})  # noqa: E501
        scenarios = ["2020-03-15", ["current", "2020-03-15"]]
        with mock.patch('FedPy.FOMC_Statement.statements.Statements.fetch_data') as patched:
            patched.return_value = demo
            for scenario in scenarios:
                if isinstance(scenario, str):
                    result = Statements(str(scenario)).to_DataFrame()
                else:
                    result = Statements(list(scenario)).to_DataFrame()
                self.assertIsInstance(result, pd.DataFrame)

    def test_to_dict(self) -> None:
        """Test method responsible for converting data to Dictionary."""
        scenarios = ["2020-03-03", ["previous", "2020-03-15"]]
        with mock.patch('FedPy.FOMC_Statement.statements.Statements.fetch_data') as patched:
            patched.return_value = json.dumps({"2021-11-03": "The Federal Reserve is committed to using its full range of tools to support the U.S. economy in this challenging time, thereby promoting its maximum employment and price stability goals.\n\n      With progress on vaccinations and strong policy support, indicators of economic activity and employment have continued to strengthen. The sectors most adversely affected by the pandemic have improved in recent months, but the summer's rise in COVID-19 cases has slowed their recovery. Inflation is elevated, largely reflecting factors that are expected to be transitory. Supply and demand imbalances related to the pandemic and the reopening of the economy have contributed to sizable price increases in some sectors. Overall financial conditions remain accommodative, in part reflecting policy measures to support the economy and the flow of credit to U.S. households and businesses.\n      \n      The path of the economy continues to depend on the course of the virus. Progress on vaccinations and an easing of supply constraints are expected to support continued gains in economic activity and employment as well as a reduction in inflation. Risks to the economic outlook remain.\n      \n      The Committee seeks to achieve maximum employment and inflation at the rate of 2 percent over the longer run. With inflation having run persistently below this longer-run goal, the Committee will aim to achieve inflation moderately above 2 percent for some time so that inflation averages 2 percent over time and longer term inflation expectations remain well anchored at 2 percent. The Committee expects to maintain an accommodative stance of monetary policy until these outcomes are achieved. The Committee decided to keep the target range for the federal funds rate at 0 to 1/4 percent and expects it will be appropriate to maintain this target range until labor market conditions have reached levels consistent with the Committee's assessments of maximum employment and inflation has risen to 2 percent and is on track to moderately exceed 2 percent for some time. In light of the substantial further progress the economy has made toward the Committee's goals since last December, the Committee decided to begin reducing the monthly pace of its net asset purchases by $10 billion for Treasury securities and $5 billion for agency mortgage-backed securities. Beginning later this month, the Committee will increase its holdings of Treasury securities by at least $70 billion per month and of agency mortgage backed securities by at least $35 billion per month. Beginning in December, the Committee will increase its holdings of Treasury securities by at least $60 billion per month and of agency mortgage-backed securities by at least $30 billion per month. The Committee judges that similar reductions in the pace of net asset purchases will likely be appropriate each month, but it is prepared to adjust the pace of purchases if warranted by changes in the economic outlook. The Federal Reserve's ongoing purchases and holdings of securities will continue to foster smooth market functioning and accommodative financial conditions, thereby supporting the flow of credit to households and businesses.\n      \n      In assessing the appropriate stance of monetary policy, the Committee will continue to monitor the implications of incoming information for the economic outlook. The Committee would be prepared to adjust the stance of monetary policy as appropriate if risks emerge that could impede the attainment of the Committee's goals. The Committee's assessments will take into account a wide range of information, including readings on public health, labor market conditions, inflation pressures and inflation expectations, and financial and international developments.\n      \n      Voting for the monetary policy action were Jerome H. Powell, Chair; John C. Williams, Vice Chair; Thomas I. Barkin; Raphael W. Bostic; Michelle W. Bowman; Lael Brainard; Richard H. Clarida; Mary C. Daly; Charles L. Evans; Randal K. Quarles; and Christopher J. Waller."})  # noqa: E501
            for scenario in scenarios:
                if isinstance(scenario, str):
                    result = Statements(str(scenario)).to_dict()
                else:
                    result = Statements(list(scenario)).to_dict()
                self.assertIsInstance(result, dict)


if __name__ in '__main__':
    unittest.main()
